<?php
$some = 'some';
$debug = false;
$database = 0;
$mailboxes = '';
$directories = null;
$admin = [];
$mailer = [];
$fetcher = [];
$cleaner = [
    'key_int' => 0,
    'key_bool' => false,
    'key_string' => ''
];
