<!DOCTYPE html>
<html lang="en">
<head>
	<title>DMARC Reports - Obsolete Page</title>
	<meta charset="UTF-8">
	<meta name="referrer" content="same-origin">
</head>
<body>
	<h1>DMARC Reports</h1>
	<p>This page is obsolete. Please change the path to the html root directory in your web server settings.</p>
	<p>You can follow this <a href="public/" title="DMARC Reports">link</a> to continue.</p>
</body>
</html>
