[role="toolbar"] {
	display: flex;
	flex-direction: row;
	position: sticky;
	top: 0;
	min-height: 2.5em;
	padding: 0 1em;
	align-items: center;
}
[role="toolbar"] button {
	height: 1.75em;
	width: 1.75em;
	padding: 2px;
	border: 0;
	background-color: transparent;
	cursor: pointer;
}
.toolbar-btn:hover, [role="toolbar"] button:hover {
	color: var(--color-fg-accent);
	background-color: var(--color-bg-hover);
}
[role="toolbar"] button svg {
	display: block;
	width: 70%;
	height: 70%;
	margin: auto;
}
[role="toolbar"] button .popup-label, .hint-block button .popup-label {
	position: absolute;
	left: -200vw;
}
.spacer-left {
	margin-left: auto;
}
.table {
	display: table;
	empty-cells: show;
}
.table-header {
	display: table-header-group;
	font-weight: bold;
}
.table-body {
	display: table-row-group;
}
.table-row {
	display: table-row;
}
.table-cell {
	display: table-cell;
}
.colspanned {
	position: relative;
}
.colspanned .table-cell:first-child {
	position: absolute;
	left: 0;
	right: 0;
}
.dialog {
	display: inline-block;
	position: fixed;
	top: 0;
	left: 0;
	right: 0;
	bottom: 0;
}
.dialog .container {
	display: flex;
	flex-direction: column;
	height: 100%;
	text-align: left;
	white-space: normal;
	background-color: var(--color-bg-dialog);
}
.dialog form {
	margin: auto 0;
	padding: 1em;
	overflow: auto;
}
.hint-block {
	display: flex;
}

.hint-block button {
	width: 1.2em;
	height: 1.2em;
	margin: auto;
	padding: 0;
	color: var(--color-fg-muted);
	line-height: 0;
	border-radius: .6em;
	background-color: transparent;
	cursor: pointer;
}
.hint-block button:hover {
	background-color: var(--color-bg-hover);
}
#stat-block ul {
	position: relative;
}
.hint-content {
	position: absolute;
	display: block;
	top: 150%;
	left: 50%;
	font-family: "Noto Sans", "Open Sans", sans-serif;
	padding: .75em;
	min-width: 10em;
	min-height: 8em;
	border: 1px solid var(--color-br-strong);
	border-radius: 5px;
	background-color: var(--color-bg-dialog);
	box-shadow: 0 0 6px var(--color-sh-default);
	transform: translateX(-50%);
	z-index: 1;
}
.hint-content::after, .hint-content::before {
	content: "";
	display: block;
	position: absolute;
	left: 50%;
	width: 0;
	border-width: 0 8px 8px;
	border-style: solid;
	transform: translateX(-50%);
}
.hint-content::after {
	top: -7px;
	border-color: var(--color-bg-dialog) transparent;
}
.hint-content::before {
	top: -9px;
	border-color: var(--color-br-strong) transparent;
}
.hint-content h4 {
	margin-bottom: 1em;
	padding-bottom: 1em;
	border-bottom: 1px dotted var(--color-br-strong);
}
.hint-content h5 {
	margin: 1em 0;
}
.hint-content h4, .hint-content h5 {
	font-size: 85%;
	text-align: center;
}
.hint-content ul {
	display: flex;
	flex-direction: column;
	gap: .5em;
}
.hint-block .hint-content li {
	display: flex;
	gap: 1em;
	align-items: end;
}
.hint-content li span:first-child {
	flex: 1;
	text-align: left;
	white-space: nowrap;
}
.hint-content .spinner {
	margin-top: 1em;
}
.titled-input {
	display: flex;
	flex-direction: column;
	row-gap: .7em;
}
.titled-input > label {
	display: flex;
	flex-direction: column;
	row-gap: .2em;
}
.titled-input > label > select {
	width: 100%;
}
.dialog-buttons {
	display: flex;
	flex-flow: column;
	justify-content: center;
	gap: 1em;
	margin: 1em 0 0 0;
}
form.vertical-content .dialog-buttons {
	margin: 0;
}
@media screen and (min-width:35em) {
	.dialog {
		position: static;
		max-width: min(45em, 98%);
	}
	.dialog .container {
		border: 1px solid #888;
	}
	.report-dialog .dialog {
		min-width: 28em;
	}
	.titled-input {
		display: table;
		margin: -.5em 0;
		border-spacing: 1em .5em;
	}
	.titled-input > label {
		display: table-row;
	}
	.titled-input > label > * {
		display: table-cell;
		vertical-align: middle;
	}
	fieldset.titled-input {
		margin-bottom: 0;
	}
	.dialog-buttons {
		flex-flow: row wrap;
	}
}
.dialog-overlay {
	display: flex;
	position: fixed;
	top: 0;
	left: 0;
	width: 100%;
	height: 100%;
	overflow: auto;
	background-color: rgba(0,0,0,0.3);
	align-items: center;
	justify-content: center;
	white-space: nowrap;
	z-index: 100;
}
.dialog .title {
	display: flex;
	align-items: center;
	justify-content: center;
	min-height: 2em;
	background-color: var(--color-bg-title);
	white-space: nowrap;
}
.dialog .title-text {
	margin: auto;
	padding-left: 0.7em;
}
.dialog .close-btn {
	margin-right: 0.1em;
	color: var(--color-fg-default);
	font-size: 1.2rem;
	font-weight: 700;
	opacity: 0.5;
	text-shadow: 0 1px 0 #fff;
	cursor: pointer;
	border: 0;
	background-color: transparent;
}
.dialog-buttons button {
	min-width: 7em;
}
.table-cell.sortable {
	cursor: pointer;
}
.sortable.arrows::before, .sortable.arrows::after {
	content: '';
	border-style: solid;
	border-width: 6px 5px;
	border-color: transparent;
	height: 0;
	position: absolute;
	right: 0.6em;
	width: 0;
}
.sortable.arrows::before {
	border-bottom-color: #bbb;
	top: 0;
}
.sortable.arrows::after {
	border-top-color: #bbb;
	bottom: 0;
}
.sortable.arrows.sorted-asc::before {
	border-bottom-color: #777;
}
.sortable.arrows.sorted-des::after {
	border-top-color: #777;
}
multi-select {
	position: relative;
}
multi-select.active .multiselect-wrapper {
	border-bottom-left-radius: 0;
	border-bottom-right-radius: 0;
}
.multiselect-wrapper {
	display: flex;
	align-items: start;
}
.multiselect-input {
	display: flex;
	flex-direction: column;
	gap: .3em;
	flex: 1;
}
.multiselect-tags {
	display: flex;
	flex-flow: row wrap;
	gap: .3em;
}
.multiselect-tags:empty {
	display: none;
}
.multiselect-tag {
	display: flex;
	max-width: 15em;
	color: #fff;
	background-color: var(--color-bg-selected2);
	border-radius: 5px;
}
.multiselect-tag>span {
	padding: 0 .25em 0 .5em;
	flex: 1;
	overflow-x: hidden;
	text-overflow: ellipsis;
}
.multiselect-tag>i {
	display: inline-block;
	border-radius: 5px;
	width: 1.35em;
	color: #020;
	text-align: center;
	transition: all .2s ease;
}
multi-select:not([disabled]) .multiselect-tag>i:hover {
	color: #fff;
	background-color: var(--color-bg-hover2);
}
multi-select:not([disabled]) .multiselect-tag>i, multi-select:not([disabled]) .multiselect-select {
	cursor: pointer;
}
multi-select[disabled] .multiselect-tags *, multi-select[disabled] .multiselect-select {
	opacity: .5;
}
.multiselect-tag>i::before {
	content: "\D7";
	font-weight: bold;
}
.multiselect-search {
	position: absolute;
	padding: 0;
	margin: 0 0 0 .2em;
	width: 0;
	font-size: 100%;
	border: 0;
	background-color: transparent;
}
.multiselect-search.active:not([disabled]) {
	position: static;
	width: 100%;
}
.multiselect-search:focus-visible {
	outline: none;
}
.multiselect-select {
	position: relative;
	width: 1em;
	height: 1.4em;
	margin: 0 -.25em 0 .5em;
	text-align: center;
	transition: transform .2s ease;
}
.multiselect-select[aria-expanded="true"] {
	transform: rotate(180deg);
}
.multiselect-select::before {
	content: "";
	position: relative;
	top: 65%;
	border-style: solid;
	border-width: 5px 5px 0;
	border-color: var(--color-fg-default) transparent transparent;
}
.multiselect-options {
	position: absolute;
	padding: 0;
	box-sizing: border-box;
	top: 100%;
	left: 0;
	width: 100%;
	max-height: 10em;
	overflow-y: auto;
	border-top: none;
	border-top-left-radius: 0;
	border-top-right-radius: 0;
}
.multiselect-options li {
	padding: .3em .5em;
	user-select: none;
}
.multiselect-options li[aria-selected="true"] {
	font-weight: bold;
}
.multiselect-options li.focused {
	background-color: var(--color-bg-selected2);
}
.multiselect-options li[aria-selected="true"].focused {
	background-color: var(--color-bg-deleted);
}
.spinner {
	display: inline-block;
}
.spinner > div {
	display: inline-block;
	background-color: currentColor;
	width: 4px;
	height: 1em;
	margin: 0 2px;
	animation: line-scale 1.1s infinite ease;
	text-align: center;
}
.spinner > div:nth-child(1) {
	animation-delay: -1.2s;
}
.spinner > div:nth-child(2) {
	animation-delay: -1.1s;
}
.spinner > div:nth-child(3) {
	animation-delay: -1s;
}
.spinner > div:nth-child(4) {
	animation-delay: -.9s;
}
.spinner > div:nth-child(5) {
	animation-delay: -.8s;
}
@keyframes line-scale {
	0%, 40%, 100% {
		transform: scaleY(.4);
	}
	20% {
		transform: scaleY(1);
	}
}
