#main-menu-block {
	float: left;
	display: block;
	width: 25px;
	height: 22px;
	margin: 0.2rem;
	background-color: var(--color-bg-default);
}
#main-menu-toggle {
	display: none;
}
#main-menu-block .button-decor, #main-menu-block .button-decor::after, #main-menu-block .button-decor::before {
	width: 25px;
	height: 4px;
	background-color: #666;
	border-radius: 3px;
	border-width: 0;
	backface-visibility: hidden;
	transition: all 300ms cubic-bezier(0.86, 0, 0.07, 1);
	z-index: 2;
}
#main-menu-block .button-decor::after, #main-menu-block .button-decor::before {
	content: "";
	position: absolute;
	left: 0;
}
#main-menu-block .button-decor {
	position: relative;
	top: 4px;
}
#main-menu-block .button-decor::after {
	top: 18px;
}
#main-menu-block .button-decor::before {
	top: 9px;
}
#main-menu-block .toggle-button {
	display: block;
	position: relative;
	top: -15%;
	left: -15%;
	width: 130%;
	height: 130%;
	cursor: pointer;
	box-sizing: border-box;
	z-index: 3;
}
#main-menu-toggle:checked ~ .button-decor {
	top: 13px;
	transform: rotate(45deg);
}
#main-menu-toggle:checked ~ .button-decor::after {
	top: 0;
	transform: rotate(90deg);
}
#main-menu-toggle:checked ~ .button-decor::before {
	top: 0;
	transform: rotate(0);
}
.menu-box {
	display: block;
	position: fixed;
	visibility: hidden;
	top: 0;
	left: -100%;
	bottom: 0;
	margin: 0;
	min-width: 16rem;
	padding: 3rem 0;
	overflow: auto;
	list-style: none;
	background-color: var(--color-bg-header);
	box-shadow: 3px 0px 6px var(--color-sh-default);
	transition: all 300ms cubic-bezier(0.86, 0, 0.07, 1);
	z-index: 1;
}
.menu-box ul {
	list-style: none;
	padding: 0;
}
.menu-box > li {
	border-top: 1px solid var(--color-br-menu);
}
.menu-box > li:last-child {
	border-bottom: 1px solid var(--color-br-menu);
}
.menu-box li li {
	background-color: var(--color-bg-default);
	border-bottom: 1px solid var(--color-br-submenu);
}
.menu-box li a, .menu-box li span {
	display: block;
	padding: 0.5rem 1.5rem;
	color: var(--color-fg-menu);
	font-size: 125%;
	text-decoration: none;
	cursor: pointer;
}
.menu-box a:hover {
	color: #fff;
	background-color: #c45;
}
#main-menu-toggle:checked ~ .menu-box {
	visibility: visible;
	left: 0;
}
.menu-box li span::after {
	content: "";
	width: 10px;
	height: 10px;
	margin-top: 9px;
	border-top: 3px solid #444;
	border-right: 3px solid #444;
	float: right;
	border-top-right-radius: 3px;
	transform: rotate(315deg);
}
.menu-box [role="menuitem"], .toggle-button {
	border: 1px solid transparent;
}
.menu-box [role="menuitem"]:focus-visible, .toggle-button:focus-visible {
	border-color: var(--color-br-focused);
}
.menu-box [role="menuitem"][aria-expanded="false"]:after {
	margin: 3px;
	transform: rotate(135deg);
}
.menu-box [role="menuitem"][aria-expanded="false"] + [role="menu"] {
	display: none;
}
.menu-box .about {
	position: absolute;
	top: 0;
	right: 0;
	font-size: 75%;
	border: none;
}
.menu-box .about a {
	padding: .8rem;
	text-decoration: underline;
}
[role="menu"] [aria-current="page"] {
	border-left: 3px solid #c45;
}
