@media (prefers-color-scheme: light) {
	:root {
		--color-fg-default:   #000;
		--color-fg-disabled:  #666;
		--color-fg-muted:     #666;
		--color-fg-accent:    #0969da;
		--color-fg-custom:    #666;
		--color-fg-menu:      #333;
		--color-bg-default:   #fff;
		--color-bg-toolbar:   #f7f7f7;
		--color-bg-header:    #eee;
		--color-bg-title:     #ddd;
		--color-bg-dialog:    #fff;
		--color-bg-selected:  #fff;
		--color-bg-selected2: #0a0;
		--color-bg-focused:   #ccf;
		--color-bg-deleted:   #f44;
		--color-bg-button:    #f0f0f0;
		--color-bg-dbutton:   #f4f4f4;
		--color-bg-hover:     #d8d8d8;
		--color-bg-hover2:    #080;
		--color-bg-input:     #fff;
		--color-br-default:   #ccc;
		--color-br-weak:      #eee;
		--color-br-strong:    #777;
		--color-br-focused:   #16d;
		--color-br-selected:  #66f;
		--color-br-input:     rgba(0,0,0,0.5);
		--color-br-disabled:  #e0e0e0;
		--color-br-menu:      #ccc;
		--color-br-submenu:   #eee;
		--color-dg-wait:      #080;
		--color-st-txtblue:   #00f;
		--color-st-txtgreen:  #080;
		--color-st-txtred:    #f00;
		--color-nt-error:     #f88;
		--color-nt-warning:   #ff8;
		--color-nt-inform:    #8f8;
		--color-sh-default:   rgba(0,0,0,0.5);
		--color-lk-default:   #00e;
		--color-lk-visited:   #551a8b;
	}
}
@media (prefers-color-scheme: dark) {
	:root {
		--color-fg-default:   #c9d1d9;
		--color-fg-disabled:  #484f58;
		--color-fg-muted:     #8b949e;
		--color-fg-accent:    #2f81f7;
		--color-fg-custom:    #aaadbb;
		--color-fg-menu:      #9ca6b1;
		--color-bg-default:   #0d1117;
		--color-bg-toolbar:   #11151e;
		--color-bg-header:    #161b22;
		--color-bg-title:     #262b32;
		--color-bg-dialog:    #111a24;
		--color-bg-selected:  #161b22;
		--color-bg-selected2: #165925;
		--color-bg-focused:   #161b33;
		--color-bg-deleted:   #4a121e;
		--color-bg-button:    #21262d;
		--color-bg-dbutton:   #21262d;
		--color-bg-hover:     #30363d;
		--color-bg-hover2:    #080;
		--color-bg-input:     #0d1117;
		--color-br-default:   #30363d;
		--color-br-weak:      #21262b;
		--color-br-strong:    #40464d;
		--color-br-focused:   #1f6feb;
		--color-br-selected:  #f78166;
		--color-br-input:     rgba(240,246,252,0.1);
		--color-br-disabled:  #30363d;
		--color-br-menu:      #505860;
		--color-br-submenu:   #30363d;
		--color-dg-wait:      #c58208;
		--color-st-txtblue:   #4040ff;
		--color-st-txtgreen:  #009300;
		--color-st-txtred:    #f22;
		--color-nt-error:     #da3633;
		--color-nt-warning:   #e7a100;
		--color-nt-inform:    #3fb950;
		--color-sh-default:   rgba(255,255,255,0.3);
		--color-lk-default:   #0050c0;
		--color-lk-visited:   #606090;
	}
}

html, body, div, span, h1, h2, h3, h4, h5, ul, li, fieldset, form, label, legend,
button, select, input, table, caption, tbody, thead, tr, th, td {
	margin: 0;
	padding: 0;
	border: 0;
	outline: 0;
	font-size: 100%;
}
html {
	color: var(--color-fg-default);
	background-color: var(--color-bg-default);
}
body {
	margin: 0.5em;
	font-family: "Noto Serif", serif;
}
h2, h3, h4, h5, table, .table {
	font-family: "Noto Sans", "Open Sans", sans-serif;
}
button, select, textarea, input, .multiselect-wrapper, .multiselect-options {
	padding: 0.3em .5em;
	color: var(--color-fg-default);
	border: 1px solid var(--color-br-input);
	border-radius: 7px;
	font-family: "Noto Sans", sans-serif;
}
.multiselect-wrapper {
	padding-left: .3em;
}
button, select, .multiselect-wrapper, .multiselect-options {
	background-color: var(--color-bg-button);
}
button:hover, select:hover, .button:hover, multi-select:not([disabled]) .multiselect-wrapper:hover {
	background-color: var(--color-bg-hover);
}
multi-select {
	min-width: 15em;
}
button:disabled, select:disabled, multi-select:disabled {
	background-color: var(--color-bg-dbutton);
}
button:disabled, select:disabled, textarea:disabled, input:disabled, multi-select:disabled {
	color: var(--color-fg-disabled);
	border-color: var(--color-br-disabled);
}
a:focus-visible, input:focus-visible, select:focus-visible, textarea:focus-visible, button:focus-visible, multi-select:focus-visible {
	outline: 2px solid var(--color-fg-accent);
	outline-offset: -2px;
}
textarea, input {
	background-color: var(--color-bg-input);
}
a {
	color: var(--color-lk-default);
}
a:visited {
	color: var(--color-lk-visited);
}
h1 {
	font-size: 1.6em;
	margin: .35rem;
	margin-top: 0;
	display: inline-block;
}
h2 {
	font-size: 1.4em;
	margin: 1em 0 .2em;
}
h3 {
	font-size: 1.3em;
	margin: 1em 0 .2em;
}
h4, h5 {
	margin: 0 0 0.5rem 0;
	text-transform: uppercase;
}
h4:not(:first-child), .info-block {
	margin-top: 1rem;
}
fieldset {
	display: block;
	margin-bottom: 0.7em;
	padding: 0.7em;
	border: 1px solid var(--color-br-strong);
}
legend {
	margin: 0;
	padding: 0 0.2em;
}
hr {
	border: 0;
	border-top: 1px solid var(--color-br-menu);
}
textarea.description {
	height: 7em;
}
label>input:first-child {
	margin-right: .7em;
}
.text {
	line-height: 1.6;
}
#stat-block>ul {
	display: flex;
	justify-content: center;
	column-gap: 1em;
	margin: .2em 0;
	padding: .1em;
	list-style-type: none;
	border-top: 1px solid var(--color-br-default);
	border-bottom: 1px solid var(--color-br-default);
}
#stat-block .stat-msg {
	display: none;
}
.main-table .table-body .table-cell {
	border-top: 1px solid var(--color-br-weak);
}
.main-table .table-body .table-row.selected, .multiselect-options li[aria-selected="true"] {
	background-color: var(--color-bg-selected);
	color: var(--color-fg-default);
}
.main-table.focused .table-body .table-row.selected {
	background-color: var(--color-bg-focused);
}
.unseen, .custom-value .setting-value {
	font-weight: bold;
	color: var(--color-fg-custom);
}
@media screen and (min-width:30em) {
	h1 {
		font-size: 2em;
		margin: -.3rem .5rem 0 .5rem;
	}
}
@media screen and (min-width:35em) {
	textarea.description {
		min-width: 25em;
	}
	input[type=number] {
		width: 4em;
	}
}
@media screen and (min-width:46em) {
	#stat-block {
		float: right;
		margin-bottom: 0.5rem;
	}
	#stat-block ul {
		display: block;
		margin: 0;
		padding: 0;
		border: none;
	}
	#stat-block li div {
		display: inline-grid;
		margin: 0.5em;
		width: 6em;
		height: 6em;
		text-align: center;
		font-size: 0.7em;
		border: 1px solid var(--color-br-strong);
		border-radius: 7px;
	}
	#stat-block .stat-msg {
		display: inline;
		margin: 0 1px;
	}
	#stat-block .hint {
		display: none;
	}
	.stat-val {
		display: block;
		height: 50%;
		line-height: 180%;
		font-size: 1.8em;
		font-weight: bold;
	}
}
@media screen and (max-width:40em) {
	.root-list-block .db-type, .list-container .mailbox-host, .list-container .dbtable-size, .small-cards .table-header {
		display: none;
	}
	.domains.table, .domains .table-header, .domains .table-body, .domains .table-cell,
	.small-cards.table, .small-cards .table-body, .small-cards .table-row {
		display: block;
	}
	.small-cards .table-cell {
		display: flow-root;
	}
	.domains .table-row {
		display: grid;
		grid-template-columns: 2em 1fr 1fr;
	}
	.domains .table-cell.descr {
		grid-column: 1 / 4;
		grid-row: 2;
		border: none;
		text-align: center;
		border-top: none;
	}
	.domains .table-body .table-cell.descr {
		border-left: 3px solid #0000;
	}
	.domains .table-body .table-row.focused .table-cell.descr, .small-cards .table-body .table-row.focused {
		border-left: 3px solid var(--color-br-selected);
	}
	.small-cards .table-body .table-row {
		border-left: 3px solid #0000;
	}
	.domains .table-body .table-cell {
		max-width: none;
	}
	.small-cards .table-body .table-cell {
		text-align: right;
		max-width: none;
		margin: 0 1em;
	}
	.small-cards .table-body .table-cell > div {
		margin-left: auto;
	}
	.small-cards .table-body .table-cell::before {
		content: attr(data-label);
		float: left;
		text-align: left;
		font-weight: bold;
		font-style: normal;
		padding-right: .5em;
	}
	.small-cards .table-body .table-cell:first-child {
		padding-top: 1em;
	}
	.small-cards .table-body .table-cell:last-child {
		border-bottom: 2px solid var(--color-br-weak);
	}
	.small-cards .table-body .table-row .table-cell {
		border-left: none !important;
	}
	.domains .table-body .table-row.selected, .domains.focused .table-body .table-row.selected,
	.small-cards .table-body .table-row.selected, .small-cards.focused .table-body .table-row.selected {
		background-color: unset;
	}
	.small-cards .table-row.colspanned .table-cell {
		text-align: center;
		border: none !important;
	}
	.custom-value .setting-value {
		font-weight: normal;
	}
}
.bottom-border {
	border-bottom: 1px solid var(--color-br-strong);
}
.popup-menu ul {
	display: block;
	margin: 0;
	padding: 0;
	list-style-type: none;
}
#stat-block > ul > li {
	margin: 0;
	padding: 0;
	display: inline-flex;
}
.disabled {
	color: #888;
	cursor: default;
	pointer-events: none;
}
.round-border, .report-result {
	border: 1px solid var(--color-br-strong);
	border-radius: 7px;
}
.wait-message {
	margin: 0.3em;
}
p.wait-message, p.error-message {
	margin: 0;
}
.panel-container .wait-message, .panel-container .error-message, .report-records .nodata {
	margin: 1em;
}
.dialog .wait-message {
	color: var(--color-dg-wait);
	text-align: center;
}
.dialog .error-message {
	text-align: center;
}
.hidden, body:not(.level-admin) .level-admin, body:not(.level-manager) .level-manager {
	display: none !important;
}
#main-block {
	clear: both;
}
.error-message {
	color: var(--color-st-txtred);
	margin: 0.3rem;
	font-style: normal;
}
.page-container, .subtable {
	border: 1px solid var(--color-br-weak);
	border-radius: 7px;
}
.page-container {
	display: flex;
	flex-direction: column;
	position: fixed;
	top: 6rem;
	left: .5rem;
	right: .5em;
	bottom: .5em;
	overflow: hidden;
}
.table-wrapper {
	flex: 1;
	overflow: auto;
}
.main-table {
	color: var(--color-fg-muted);
	text-align: left;
	border-spacing: 0;
	width: 100%;
}
.subtable {
	font-size: 90%;
}
.main-table .table-header {
	position: sticky;
	top: 0;
}
[role="toolbar"] {
	background-color: var(--color-bg-toolbar);
}
[role="toolbar"], [role="toolbar"] button {
	color: var(--color-fg-muted);
}
[role="toolbar"] span {
	font-size: 85%;
}
.main-table .table-header .table-cell {
	position: relative;
	background-color: var(--color-bg-header);
}
.main-table .table-cell {
	padding: .3em 1em;
}
.main-table .table-body .table-row:not(.noninteractive) {
	cursor: pointer;
}
.main-table .table-body .table-row:not(.noninteractive):hover, .main-table .table-row.virtual-item:hover .table-cell {
	background-color: var(--color-bg-selected);
}
.focused {
	outline: none;
}
.main-table .table-body .table-cell:first-child {
	border-left: 3px solid #0000;
}
.main-table .table-body .table-row.focused .table-cell:first-child {
	border-left: 3px solid var(--color-br-selected);
}
.summary-report {
	overflow-y: auto;
}
.summary-report hr {
	margin: 2em 0;
}
.report-table {
	font-size: 90%;
	border-spacing: 0;
	border-collapse: collapse;
}
.custom-value .setting-value, .report-table caption {
	color: var(--color-fg-muted);
	font-style: italic;
}
.report-table th {
	min-width: 4em;
	padding: .3em .6em;
}
.report-table td {
	padding: .3em;
	text-align: right;
}
.report-table td:first-child {
	text-align: left;
	padding-right: 1em;
}
.report-table td, .report-table th {
	border: 1px solid var(--color-br-default);
}
.state-red .state-text {
	color: var(--color-st-txtred);
}
.state-red .state-background {
	background-color: #f00;
}
.state-green .state-text {
	color: var(--color-st-txtgreen);
}
.state-green .state-background {
	background-color: #080;
}
.state-yellow .state-text {
	color: #f80;
}
.state-yellow .state-background {
	background-color: #f80;
}
.state-gray .state-text {
	color: #888;
}
.state-gray .state-background {
	background-color: #888;
}
.state-blue .state-text {
	color: var(--color-st-txtblue);
}
.state-blue .state-background {
	background-color: #00f;
}
.panel-container {
	margin: 0.5rem 0;
	padding: 1rem;
}
.block-item-column {
	display: flex;
	margin: .2rem 0;
}
.block-item-column .value {
	margin-left: 0.5em;
}
.root-list-block, .block-list-item {
	display: flex;
	gap: .5rem;
	align-items: center;
	padding: 0 .75rem;
}
.root-list-block {
	min-height: 4rem;
	font-size: 1.3rem;
	cursor: pointer;
}
.block-list-item {
	height: 3.25rem;
	background-color: var(--color-bg-header);
	margin: 0 0 1rem 0;
}
.list-container {
	margin: 1rem 0.8rem 0 1.5rem;
}
.title-item-wrap, .db-type, .db-location, .dbtable-records, .dbtable-size, .mailbox-host, .mailbox-location {
	flex: 1;
}
.message-item, .directory-location {
	flex: 2;
}
.status-indicator {
	width: 1rem;
	min-width: 1rem;
	margin: auto 0.5rem auto 0;
	height: 1rem;
	border-radius: 50%;
	background: radial-gradient(circle farthest-corner at 35% 35%, rgba(255, 255, 255, 0.8), rgba(255, 255, 255, 0));
}
.root-list-block .status-indicator {
	width: 1.2rem;
	height: 1.2rem;
}
.db-menu-button {
	position: relative;
	width: 1.7em;
}
.db-menu-button span[role="button"] {
	font-weight: bold;
	color: #888;
	user-select: none;
}
.db-menu-button .button {
	width: 100%;
	text-align: center;
	border-radius: 3px;
}
.db-menu-button .button span {
	position: relative;
	display: block;
	top: -15%;
}
.source-check-button {
	margin-right: 0.7em;
}
.popup-menu {
	position: absolute;
	top: 130%;
	left: -5.3em;
	width: 7em;
	text-align: center;
	background-color: var(--color-bg-default);
}
.popup-menu::after {
	top: -0.3em;
	border-color: var(--color-bg-default) transparent;
}
.popup-menu::before {
	top: -0.4em;
	border-color: var(--color-br-strong) transparent;
}
.popup-menu::after, .popup-menu::before {
	content: "";
	display: block;
	position: absolute;
	bottom: auto;
	right: auto;
	left: 5.4em;
	width: 0;
	border-width: 0 8px 8px;
	border-style: solid;
}
.popup-menu li {
	margin: 0.2em 0 0.2em 0;
}
.popup-menu li:hover {
	color: #00f;
}
.report-result {
	display: inline-block;
	padding: 0.2em;
	margin: 0 0.2em 0 0.2em;
}
.main-table .report-result {
	font-size: 75%;
}
.report-result-pass {
	color: var(--color-st-txtgreen);
}
.report-result-unknown {
	color: #888;
}
.report-result-fail {
	color: var(--color-st-txtred);
}
.report-modal {
	position: fixed;
	top: 0;
	left: 0;
	width: 100%;
	height: 100%;
	margin: 0;
	padding: 0;
	background-color: var(--color-bg-default);
	text-align: center;
	transition: all 600ms cubic-bezier(0.86, 0, 0.07, 1);
	overflow: auto;
}
.report-modal > .report-header {
	position: sticky;
	top: 0;
	padding: 1rem;
	cursor: pointer;
	border-bottom: 1px solid var(--color-br-default);
	background-color: var(--color-bg-title);
	z-index: 1;
}
.report-modal > .report-header > .header-text {
	padding: 0.5rem;
	cursor: auto;
}
.report-record > .header {
	position: relative;
	display: flex;
	flex-flow: row wrap;
	gap: .5em;
	justify-content: center;
	padding: 0.4em 0;
	border-bottom: 1px solid var(--color-br-default);
	border-radius: 7px 7px 0 0;
	background-color: var(--color-bg-header);
}
.report-modal > .body {
	padding: 1rem;
}
.report-modal.report-hidden {
	top: 100%;
}
.report-modal .close-btn {
	position: relative;
	display: inline-block;
	padding: 0.7em 1.5em;
	color: var(--color-fg-default);
	background-color: var(--color-bg-button);
	border: 1px solid #888;
	border-radius: 30px;
	cursor: pointer;
	transition: all .3s ease;
}
.report-modal .content {
	width: 50rem;
	max-width: 100%;
	margin: 0 auto;
}
.report-modal .close-btn:hover {
	box-shadow: 0 3px 15px var(--color-sh-default);
}
.report-records {
	position: relative;
	margin-top: 1rem;
	padding-top: 1rem;
	border-top: 1px solid var(--color-br-default);
}
.report-record {
	margin: 0 0 1rem 0;
	border-color: var(--color-br-default);
}
.report-item {
	display: inline-flex;
	width: 98%;
	padding: .3em 0;
}
.report-record .report-item {
	min-height: 1.9em;
}
.report-record .report-item:not(:last-child) {
	border-bottom: 1px solid var(--color-br-weak);
}
.report-item .title {
	min-width: 30%;
}
.value.rows {
	display: flex;
	flex-direction: column;
	row-gap: .3em;
}
.report-item .value {
	overflow-x: hidden;
}
.report-item .fqdn, .options-block .fqdn {
	word-wrap: anywhere;
}
.report-item .title, .report-item .value {
	text-align: left;
	margin: auto 0;
}
.table-cell.fqdn, .table-cell.orgname, .table-cell.report-id, .table-cell.setting-value, .hint-content li>span:nth-child(2) {
	max-width: 15em;
	overflow: hidden;
	text-overflow: ellipsis;
	white-space: nowrap;
}
.options-button {
	color: #888;
	font-size: 80%;
	margin-left: 0.3em;
	cursor: pointer;
}
.buttons-block > button {
	margin: 0;
}
button.options-button {
	color: inherit;
}
.options-button:hover, .options-button.active, [role="toolbar"] button.active {
	color: #080;
}
.options-block h2 {
	margin: 0;
	font-size: 1.2em;
}
.options-block ul {
	margin: 1em 0;
	list-style-type: none;
}
.buttons-block {
	display: flex;
	gap: .5em;
	margin-top: .5em;
}
.table-cell.cell-status {
	width: 1em;
}
.virtual-item, .nodata {
	color: #888;
	font-style: italic;
}
.left-titled {
	display: grid;
	grid-template-columns: auto 1fr;
	row-gap: .3em;
	column-gap: 1em;
	align-items: center;
}
.vertical-content {
	display: flex;
	flex-direction: column;
	gap: 1em;
}
.vertical-content span.value {
	padding: .3em 0;
	border: 1px solid transparent;
}
.vertical-content > * {
	margin-top: 0;
	margin-bottom: 0;
}
.about h2 {
	letter-spacing: .2em;
}
.about .vertical-content, .verification .vertical-content {
	max-width: 30em;
}
.verification textarea {
	height: 4em;
}
.warn-block {
	color: #000;
	padding: .5em;
	text-align: center;
	background-color: var(--color-nt-warning);
}
.toolbar-btn {
	position: absolute;
	right: 0;
	padding: .1em .5em;
	border: none;
	border-radius: unset;
	background-color: unset;
	cursor: pointer;
}
.toolbar-btn::before, .toolbar-btn::after {
	content: "";
	position: absolute;
	box-sizing: border-box;
	width: 1em;
	height: 100%;
	border-top: 1px solid var(--color-br-input);
	border-bottom: 1px solid var(--color-br-input);
}
.toolbar-btn::before {
	top: 0;
	left: 0;
	border-left: 1px solid var(--color-br-input);
}
.toolbar-btn::after {
	right: 0;
	bottom: 0;
	border-right: 1px solid var(--color-br-input);
}
